'demo program to emulate Sparkfun Nordic KEY FOB  WRL-08602
'sends a key code and 16 bit counter every second 
'using Sparkfun Nordic trasceiver WRL-00705
#picaxe 20m2
'#no_data
#com 1
#terminal 4800

' -----[ I/O Definitions ]-------------------------------------------------
Symbol CE   = C.0                        ' Enable        -> Nordic CE
Symbol CS   = C.1                        ' Chip Select   -> Nordic CSN
symbol SCK  = C.2                        ' SPI clock     -> Nordic SCK 
symbol MOSI = C.3                        ' Shiftout      -> Nordic MOSI 
symbol MISO = pinC.4                     ' Shiftin       -> Nordic MISO
symbol IRQ  = C.5                        ' Interrupt     -> Nordic IRQ 


' ~~~~~ SPI SYMBOL DEFINITIONS ~~~~~
' Required for all routines. Change pin numbers/bits as required.
' Uses variables b7-b13 (i.e. b7,w4,w5,w6). If only using 8 bits
' all the word variables can be safely changed to byte variables.
'

symbol counter = b7                     ' variable used during loop
symbol mask = w4                        ' bit masking variable
symbol var_in = w5                      ' data variable used durig shiftin
symbol var_out = w6                     ' data variable used during shiftout
symbol bits = 8                         ' number of bits
symbol MSBvalue = 128                   ' MSBvalue
                                        '(=128 for 8 bits, 512 for 10 bits, 2048 for 12 bits)

' -----[ Constants ]-------------------------------------------------------
'Registers
Symbol  CONFIG      = $00
Symbol  EN_AA       = $01
Symbol  EN_RXADDR   = $02
Symbol  SETUP_AW    = $03
Symbol  SETUP_RETR  = $04
Symbol  RF_CH       = $05
Symbol  RF_SETUP    = $06
Symbol  Status_Reg  = $07
Symbol  OBSERVE_TX  = $08
Symbol  RPD         = $09
Symbol  RX_ADDR_P0  = $0A
Symbol  RX_ADDR_P1  = $0B
Symbol  RX_ADDR_P2  = $0C
Symbol  RX_ADDR_P3  = $0D
Symbol  RX_ADDR_P4  = $0E
Symbol  RX_ADDR_P5  = $0F
Symbol  TX_ADDR     = $10
Symbol  RX_PW_P0    = $11
Symbol  RX_PW_P1    = $12
Symbol  RX_PW_P2    = $13
Symbol  RX_PW_P3    = $14
Symbol  RX_PW_P4    = $15
Symbol  RX_PW_P5    = $16
Symbol  FIFO_Status = $17
Symbol  DYNPD       = $1C
Symbol  FEATURE     = $1D

Symbol  reg0        = %00001111    '0,MASK_RX_DR,MASK_TX_DS,MASK_MAX_RT,EN_CRC,CRCO,PWR_UP,PRIM_RX
'                                          0      0          0              1     1   1     1
Symbol  reg1        = $01    'Auto Acknowledgement  - Enable auto acknowledgement data pipe 0
Symbol  reg2        = %00000001    'Enabled RX Addresses - enable pipe 0
Symbol  reg3        = %00000011    'RX/TX Address field width - '11' = 5 bytes
Symbol  reg4        = %00111111     ' 1 msec pauza, 15 opakovani

Symbol  reg5        = $10    'RF Channel Freq = 2400 + RF_CH [MHz]
Symbol  reg6        = %00000110    ' 1 Mbps, max power 0 dbM
' $07    'RF Setup Register - Set RF Data Rate to 1Mbps & Set RF output power in TX mode '11' ? 0dBm

'Commands
Symbol  R_REGISTER            = %00000000 '+ reg
Symbol  W_REGISTER            = %00100000 '+ reg
Symbol  R_RX_PAYLOAD          = %01100001
Symbol  W_TX_PAYLOAD          = %10100000
Symbol  FLUSH_TX              = %11100001
Symbol  FLUSH_RX              = %11100010
Symbol  REUSE_TX_PL           = %11100011
Symbol  R_RX_PL_WID           = %01100000
Symbol  W_ACK_PAYLOAD         = %10101000 '+ PIPE
Symbol  W_TX_PAYLOAD_NOACK    = %10110000
Symbol  NOOP                  = %11111111

'Misc constants
Symbol  PTX = %11111110  
Symbol  payloadsize = 1           'Number of bytes in RX payload in data pipe 0

' bylo tu: Symbol  statusReset = %01000000   'Data Ready RX FIFO interrupt (bit 6) - Write 1 to clear bit.
' 0, MASK_RX_DR, MASK_TX_DS,  MASK_MAX_RT
Symbol  statusReset = %01110000


Symbol adr1 = $37
Symbol adr2 = $51
Symbol adr3 = $fa
Symbol adr4 = $21
Symbol adr5 = $56

' -----[ Variables ]-------------------------------------------------------
Symbol  regval  = b0
Symbol  idx     = b2
Symbol  idx2    = b4

symbol temp     = b25

 EEPROM 0,(reg0,reg1,reg2,reg3,reg4,reg5,reg6)

Setup:
 sertxd ("start", CR, LF)
  pause 1000
  gosub ConfigRTN     'load configuration registers
  pause 1000
  gosub DebugRTN      'display configuration registers
  pause 1000

'transmit loop
 DO
	 readtemp b.7,idx2 ; read value into b1
	sertxd (CR,LF,CR,LF,"teplota: ", #idx2,CR,LF)
	
	if idx2 > 0 and idx2 < 80 then 
		 pulsout b.2, 20000
	end if

	if idx2 > 80 and idx2 < 120 then 
		 pulsout b.0, 20000
		 pause 200
		 pulsout b.0, 20000
		 pause 200
		 pulsout b.0, 20000
		 pause 200
		 pulsout b.0, 20000
	end if
		 
	bptr = 28
	@bptrinc = idx2
	GOSUB WritePayload
	
	PAUSE 5000
 LOOP
  end


  
'------------------------------------------------------------------  
WritePayload:
  ' LOW CE       
  
gosub ReadStatus      

'Reset IRQ
sertxd ("Status_Reg",CR,LF)
  LOW CS
  var_out = W_REGISTER + Status_Reg
  gosub shiftout_MSBFirst
  var_out = statusReset
  gosub shiftout_MSBFirst
  HIGH CS

gosub ReadStatus      

sertxd ("CONFIG",CR,LF)
  READ Config,regval
  LOW CS
  var_out = W_REGISTER + Config
  gosub shiftout_MSBFirst
  var_out = regval & PTX                     'PTX - transmit mode
  gosub shiftout_MSBFirst
  HIGH CS

sertxd ("FLUSH_TX",CR,LF)
  LOW CS
  var_out = FLUSH_TX
  gosub shiftout_MSBFirst
  HIGH CS

sertxd ("W_TX_PAYLOAD",CR,LF)
  LOW CS
  var_out = W_TX_PAYLOAD
  gosub shiftout_MSBFirst
  bptr = 28
  FOR idx = 1 TO payloadsize
    var_out = @bptrinc 
    gosub shiftout_MSBFirst
  NEXT
  HIGH CS
  
  
' vlastni odesilani probehne zde
  HIGH CE
  PAUSE 500
  LOW CE

  sertxd ("CONFIG",CR,LF)
  READ Config,regval
  LOW CS
  var_out = W_REGISTER + Config
  gosub shiftout_MSBFirst
  var_out = regval                          'PRX - receive mode
  gosub shiftout_MSBFirst
  HIGH CS
  
  gosub ReadStatus
  
  mask = regval & %00100000
  if mask = %00100000  then
      sertxd( "data odeslana", CR, LF)
	pulsout b.2, 50000
  end if
           
  mask = regval & %00010000        
  if mask = %00010000 then
      sertxd( "prijemce mimo dosah", CR, LF)
	pulsout b.0, 50000
  end if
           
  RETURN  

ReadStatus:
  LOW CS
  var_out = R_REGISTER + Status_Reg
  gosub shiftout_MSBFirst
  gosub shiftin_MSB_Pre 
  regval = var_in 
  sertxd ("Status: ", #bit7,#bit6,#bit5,#bit4,#bit3,#bit2,#bit1,#bit0,CR,LF)
  HIGH CS
  RETURN

ConfigRTN:
	sertxd ("Configuration:",CR,LF)
  HIGH CS           'Chip select
  INPUT IRQ
  OUTPUT CE
  OUTPUT CS
  LOW SCK
  LOW MOSI
  HIGH CE            'disable transceiver

  output B.0
  high B.0
  output b.2
  high b.2
  
  pulsout b.0, 50000
  pulsout b.2, 50000
  pulsout b.0, 50000
  pulsout b.2, 50000


	sertxd( "EN_AA", CR, LF )
  Read EN_AA,regval
  idx=EN_AA : gosub DisplayReg
  LOW CS
  var_out = W_REGISTER + EN_AA             ' Auto Acknowledgement
  gosub shiftout_MSBFirst
  var_out = regval
  gosub shiftout_MSBFirst
  HIGH CS

	sertxd( "SETUP_RETR", CR, LF )
 Read SETUP_RETR,regval
  idx=SETUP_RETR : gosub DisplayReg
  LOW CS
  var_out = W_REGISTER + SETUP_RETR        ' Auto Retransmit Delay  
  gosub shiftout_MSBFirst
  var_out = regval
  gosub shiftout_MSBFirst
  HIGH CS

sertxd( "SETUP_AW", CR, LF )
  Read SETUP_AW,regval
  idx=SETUP_AW : gosub DisplayReg
  LOW CS
  var_out = W_REGISTER + SETUP_AW          ' RX/TX Address field width 
  gosub shiftout_MSBFirst
  var_out = regval
  gosub shiftout_MSBFirst
  HIGH CS

sertxd( "RF_SETUP", CR, LF )
  Read RF_SETUP,regval
  idx=RF_SETUP : gosub DisplayReg
  LOW CS
  var_out = W_REGISTER + RF_SETUP          ' Set RF Data Rate, etc 
  gosub shiftout_MSBFirst
  var_out = regval
  gosub shiftout_MSBFirst
  HIGH CS

sertxd( "RX_PW_P0", CR, LF )
  LOW CS
  var_out =W_REGISTER + RX_PW_P0           ' Set Number of bytes in RX payload in data pipe 0 
  gosub shiftout_MSBFirst
  var_out = payloadsize
  gosub shiftout_MSBFirst
  HIGH CS

sertxd( "RF_CH", CR, LF )
  Read RF_CH,regval
  idx=RF_CH : gosub DisplayReg
  LOW CS
  var_out = W_REGISTER + RF_CH             ' RF Channel Freq 
  gosub shiftout_MSBFirst
  var_out = regval
  gosub shiftout_MSBFirst
  HIGH CS

sertxd( "RX_ADDR_P0", CR, LF )
  LOW CS
  var_out = W_REGISTER + RX_ADDR_P0
  gosub shiftout_MSBFirst
    var_out = adr1
    gosub shiftout_MSBFirst
    var_out = adr2
    gosub shiftout_MSBFirst
    var_out = adr3
    gosub shiftout_MSBFirst
    var_out = adr4
    gosub shiftout_MSBFirst
    var_out = adr5
    gosub shiftout_MSBFirst
  HIGH CS

sertxd( "TX_ADDR", CR, LF )
  LOW CS
  var_out = W_REGISTER + TX_ADDR
  gosub shiftout_MSBFirst
    var_out = adr1
    gosub shiftout_MSBFirst
    var_out = adr2
    gosub shiftout_MSBFirst
    var_out = adr3
    gosub shiftout_MSBFirst
    var_out = adr4
    gosub shiftout_MSBFirst
    var_out = adr5
    gosub shiftout_MSBFirst
  HIGH CS

sertxd( "CONFIG", CR, LF )
  Read Config,regval
  idx=Config : gosub DisplayReg
  LOW CS
  var_out = W_REGISTER + Config
  gosub shiftout_MSBFirst
  var_out = regval
  gosub shiftout_MSBFirst
  HIGH CS

  LOW CE            'enable transceiver
  RETURN

DebugRTN:
  sertxd ("Initialized.",CR,LF)
  GOSUB ReadConfig
  GOSUB ReadRxAddr
  GOSUB readTxAddr
  RETURN

ReadConfig:
  sertxd ("Read config:",CR,LF)
  FOR idx = 0 TO 6
    LOW CS
    var_out = R_REGISTER + idx
    gosub shiftout_MSBFirst
    
    gosub shiftin_MSB_Pre 
    regval = var_in
    gosub DisplayReg
    HIGH CS
  NEXT
  RETURN

ReadRxAddr:
  LOW CS
  var_out = R_REGISTER + RX_ADDR_P0
  gosub shiftout_MSBFirst
  
  sertxd ("rxAddr P0: ")
  bptr = 28
  FOR idx = 0 TO 4
    gosub shiftin_MSB_Pre 
    @bptrinc = var_in
  NEXT
  HIGH CS
  bptr = 28
  FOR idx = 0 TO 4
    regval = @bptrinc
    gosub DisplayRegHex
    sertxd (" ")
  NEXT
  sertxd (CR,LF)
  RETURN

ReadTxAddr:
  LOW CS
  var_out = R_REGISTER + TX_ADDR
  gosub shiftout_MSBFirst
  sertxd ("txAddr: ")
  bptr = 28
  FOR idx = 0 TO 4
    gosub shiftin_MSB_Pre 
    @bptrinc = var_in
  NEXT
  HIGH CS
  bptr = 28
  FOR idx = 0 TO 4
    regval = @bptrinc
    gosub DisplayRegHex
    sertxd (" ")
  NEXT
  sertxd (CR,LF)
  RETURN
  
DisplayReg:  
  sertxd ("Register ",#idx," = ", #bit7,#bit6,#bit5,#bit4,#bit3,#bit2,#bit1,#bit0,CR,LF)
  Return  
  
DisplayRegHex:
  sertxd ("$")
  temp = regval / 16 + "0"    				'cnvrt byte to ascii
  gosub Nibble
  temp = regval & 15 + "0"    				'clear bits 7 thru 4 and convert to ascii 
Nibble:
  if temp > "9" then : temp = temp + 7 : endif  'bridge gap between "9" & "A"
  sertxd(temp)
  return  



'========================================================================
' ***** Shiftout MSB first *****
shiftout_MSBFirst:
  'sertxd ("> ", #var_out, CR, LF )
  
  for counter = 1 to bits          ' number of bits
    mask = var_out & MSBValue      ' mask MSB
    high MOSI                      ' data high
    if mask = MSBValue then skipMSB                           ' RIGHT - from the sample shiftin_out.bas   
    low MOSI                       ' data low
skipMSB: 
    pulsout SCK,1                  ' pulse clock for 10us
    var_out = var_out * 2          ' shift variable left for MSB
  next counter
return

'========================================================================
'========================================================================
' ~~~~~ SHIFTIN ROUTINES ~~~~~
' Only one of these 4 is required - see your IC requirements
' It is recommended you delete the others to save space
'========================================================================
'========================================================================
' ***** Shiftin MSB first, Data Pre-Clock *****
shiftin_MSB_Pre:
  let var_in = 0
  for counter = 1 to bits           ' number of bits
    var_in = var_in * 2             ' shift left as MSB first
    if MISO = 0 then skipMSBPre
    var_in = var_in + 1             ' set LSB if MISO = 1
skipMSBPre: 
    pulsout SCK,1                   ' pulse clock to get next data bit
  next counter
  
  'sertxd( "< ",  #var_in, CR, LF )
return


